@@ -1,6 +1,7 @@
 import asyncio
 import functools
 
+from ..abc import AbcPool
 from ..errors import (
     RedisError,
     PipelineError,
@@ -9,8 +10,6 @@ from ..errors import (
     )
 from ..util import (
     wait_ok,
-    async_task,
-    create_future,
     _set_exception,
     )
 
@@ -36,13 +35,13 @@ class TransactionsCommandsMixin:
 
     def unwatch(self):
         """Forget about all watched keys."""
-        fut = self._conn.execute(b'UNWATCH')
+        fut = self._pool_or_conn.execute(b'UNWATCH')
         return wait_ok(fut)
 
     def watch(self, key, *keys):
         """Watch the given keys to determine execution of the MULTI/EXEC block.
         """
-        fut = self._conn.execute(b'WATCH', key, *keys)
+        fut = self._pool_or_conn.execute(b'WATCH', key, *keys)
         return wait_ok(fut)
 
     def multi_exec(self):
@@ -59,8 +58,8 @@ class TransactionsCommandsMixin:
         >>> await asyncio.gather(fut1, fut2)
         [1, 1]
         """
-        return MultiExec(self._conn, self.__class__,
-                         loop=self._conn._loop)
+        return MultiExec(self._pool_or_conn, self.__class__,
+                         loop=self._pool_or_conn._loop)
 
     def pipeline(self):
         """Returns :class:`Pipeline` object to execute bulk of commands.
@@ -86,8 +85,8 @@ class TransactionsCommandsMixin:
         >>> await asyncio.gather(fut1, fut2)
         [2, 2]
         """
-        return Pipeline(self._conn, self.__class__,
-                        loop=self._conn._loop)
+        return Pipeline(self._pool_or_conn, self.__class__,
+                        loop=self._pool_or_conn._loop)
 
 
 class _RedisBuffer:
@@ -99,7 +98,7 @@ class _RedisBuffer:
         self._loop = loop
 
     def execute(self, cmd, *args, **kw):
-        fut = create_future(loop=self._loop)
+        fut = self._loop.create_future()
         self._pipeline.append((fut, cmd, args, kw))
         return fut
 
@@ -123,11 +122,11 @@ class Pipeline:
     """
     error_class = PipelineError
 
-    def __init__(self, connection, commands_factory=lambda conn: conn,
+    def __init__(self, pool_or_connection, commands_factory=lambda conn: conn,
                  *, loop=None):
         if loop is None:
             loop = asyncio.get_event_loop()
-        self._conn = connection
+        self._pool_or_conn = pool_or_connection
         self._loop = loop
         self._pipeline = []
         self._results = []
@@ -143,16 +142,17 @@ class Pipeline:
             @functools.wraps(attr)
             def wrapper(*args, **kw):
                 try:
-                    task = async_task(attr(*args, **kw), loop=self._loop)
+                    task = asyncio.ensure_future(attr(*args, **kw),
+                                                 loop=self._loop)
                 except Exception as exc:
-                    task = create_future(loop=self._loop)
+                    task = self._loop.create_future()
                     task.set_exception(exc)
                 self._results.append(task)
                 return task
             return wrapper
         return attr
 
-    def execute(self, *, return_exceptions=False):
+    async def execute(self, *, return_exceptions=False):
         """Execute all buffered commands.
 
         Any exception that is raised by any command is caught and
@@ -165,25 +165,29 @@ class Pipeline:
         self._done = True
 
         if self._pipeline:
-            return self._do_execute(return_exceptions=return_exceptions)
+            if isinstance(self._pool_or_conn, AbcPool):
+                async with self._pool_or_conn.get() as conn:
+                    return await self._do_execute(
+                        conn, return_exceptions=return_exceptions)
+            else:
+                return await self._do_execute(
+                    self._pool_or_conn,
+                    return_exceptions=return_exceptions)
         else:
-            return self._gather_result(return_exceptions)
+            return await self._gather_result(return_exceptions)
 
-    @asyncio.coroutine
-    def _do_execute(self, *, return_exceptions=False):
-        conn = yield from self._conn.get_atomic_connection()
-        yield from asyncio.gather(*self._send_pipeline(conn),
-                                  loop=self._loop,
-                                  return_exceptions=True)
-        return (yield from self._gather_result(return_exceptions))
+    async def _do_execute(self, conn, *, return_exceptions=False):
+        await asyncio.gather(*self._send_pipeline(conn),
+                             loop=self._loop,
+                             return_exceptions=True)
+        return await self._gather_result(return_exceptions)
 
-    @asyncio.coroutine
-    def _gather_result(self, return_exceptions):
+    async def _gather_result(self, return_exceptions):
         errors = []
         results = []
         for fut in self._results:
             try:
-                res = yield from fut
+                res = await fut
                 results.append(res)
             except Exception as exc:
                 errors.append(exc)
@@ -249,10 +253,8 @@ class MultiExec(Pipeline):
     """
     error_class = MultiExecError
 
-    @asyncio.coroutine
-    def _do_execute(self, *, return_exceptions=False):
+    async def _do_execute(self, conn, *, return_exceptions=False):
         self._waiters = waiters = []
-        conn = yield from self._conn.get_atomic_connection()
         multi = conn.execute('MULTI')
         coros = list(self._send_pipeline(conn))
         exec_ = conn.execute('EXEC')
@@ -260,9 +262,9 @@ class MultiExec(Pipeline):
                                 return_exceptions=True)
         last_error = None
         try:
-            yield from asyncio.shield(gather, loop=self._loop)
+            await asyncio.shield(gather, loop=self._loop)
         except asyncio.CancelledError:
-            yield from gather
+            await gather
         except Exception as err:
             last_error = err
             raise
@@ -279,7 +281,7 @@ class MultiExec(Pipeline):
                         # fut.cancel()
             else:
                 try:
-                    results = yield from exec_
+                    results = await exec_
                 except RedisError as err:
                     for fut in waiters:
                         fut.set_exception(err)
@@ -287,7 +289,7 @@ class MultiExec(Pipeline):
                     assert len(results) == len(waiters), (
                         "Results does not match waiters", results, waiters)
                     self._resolve_waiters(results, return_exceptions)
-            return (yield from self._gather_result(return_exceptions))
+            return (await self._gather_result(return_exceptions))
 
     def _resolve_waiters(self, results, return_exceptions):
         errors = []
@@ -303,7 +305,7 @@ class MultiExec(Pipeline):
     def _check_result(self, fut, waiter):
         assert waiter not in self._waiters, (fut, waiter, self._waiters)
         assert not waiter.done(), waiter
-        if fut.cancelled():     # yield from gather was cancelled
+        if fut.cancelled():     # await gather was cancelled
             waiter.cancel()
         elif fut.exception():   # server replied with error
             waiter.set_exception(fut.exception())
