@@ -1,6 +1,5 @@
 import types
 import asyncio
-import hiredis
 import socket
 from functools import partial
 from collections import deque
@@ -13,9 +12,10 @@ from .util import (
     _set_exception,
     coerced_keys_dict,
     decode,
-    async_task,
-    create_future,
+    parse_url,
     )
+from .parser import Reader
+from .stream import open_connection, open_unix_connection
 from .errors import (
     ConnectionClosedError,
     ConnectionForcedCloseError,
@@ -23,9 +23,12 @@ from .errors import (
     ProtocolError,
     ReplyError,
     WatchVariableError,
+    ReadOnlyError,
+    MaxClientsError
     )
 from .pubsub import Channel
 from .abc import AbcChannel
+from .abc import AbcConnection
 from .log import logger
 
 
@@ -41,16 +44,15 @@ _PUBSUB_COMMANDS = (
     )
 
 
-@asyncio.coroutine
-def create_connection(address, *, db=None, password=None, ssl=None,
-                      encoding=None, loop=None, timeout=None):
+async def create_connection(address, *, db=None, password=None, ssl=None,
+                            encoding=None, parser=None, loop=None,
+                            timeout=None, connection_cls=None):
     """Creates redis connection.
 
     Opens connection to Redis server specified by address argument.
-    Address argument is similar to socket address argument, ie:
-    * when address is a tuple it represents (host, port) pair;
-    * when address is a str it represents unix domain socket path.
-    (no other address formats supported)
+    Address argument can be one of the following:
+    * A tuple representing (host, port) pair for TCP connections;
+    * A string representing either Redis URI or unix domain socket path.
 
     SSL argument is passed through to asyncio.create_connection.
     By default SSL/TLS is not used.
@@ -62,20 +64,47 @@ def create_connection(address, *, db=None, password=None, ssl=None,
     Encoding argument can be used to decode byte-replies to strings.
     By default no decoding is done.
 
-    Return value is RedisConnection instance.
+    Parser parameter can be used to pass custom Redis protocol parser class.
+    By default hiredis.Reader is used (unless it is missing or platform
+    is not CPython).
+
+    Return value is RedisConnection instance or a connection_cls if it is
+    given.
 
     This function is a coroutine.
     """
     assert isinstance(address, (tuple, list, str)), "tuple or str expected"
+    if isinstance(address, str):
+        logger.debug("Parsing Redis URI %r", address)
+        address, options = parse_url(address)
+        db = options.setdefault('db', db)
+        password = options.setdefault('password', password)
+        encoding = options.setdefault('encoding', encoding)
+        timeout = options.setdefault('timeout', timeout)
+        if 'ssl' in options:
+            assert options['ssl'] or (not options['ssl'] and not ssl), (
+                "Conflicting ssl options are set", options['ssl'], ssl)
+            ssl = ssl or options['ssl']
 
     if timeout is not None and timeout <= 0:
         raise ValueError("Timeout has to be None or a number greater than 0")
 
+    if connection_cls:
+        assert issubclass(connection_cls, AbcConnection),\
+                "connection_class does not meet the AbcConnection contract"
+        cls = connection_cls
+    else:
+        cls = RedisConnection
+
+    if loop is None:
+        loop = asyncio.get_event_loop()
+
     if isinstance(address, (list, tuple)):
         host, port = address
         logger.debug("Creating tcp connection to %r", address)
-        reader, writer = yield from asyncio.wait_for(asyncio.open_connection(
-            host, port, ssl=ssl, loop=loop), timeout, loop=loop)
+        reader, writer = await asyncio.wait_for(open_connection(
+            host, port, limit=MAX_CHUNK_SIZE, ssl=ssl, loop=loop),
+            timeout, loop=loop)
         sock = writer.transport.get_extra_info('socket')
         if sock is not None:
             sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
@@ -83,45 +112,55 @@ def create_connection(address, *, db=None, password=None, ssl=None,
         address = tuple(address[:2])
     else:
         logger.debug("Creating unix connection to %r", address)
-        reader, writer = yield from asyncio.wait_for(
-            asyncio.open_unix_connection(address, ssl=ssl, loop=loop),
+        reader, writer = await asyncio.wait_for(open_unix_connection(
+            address, ssl=ssl, limit=MAX_CHUNK_SIZE, loop=loop),
             timeout, loop=loop)
         sock = writer.transport.get_extra_info('socket')
         if sock is not None:
             address = sock.getpeername()
-    conn = RedisConnection(reader, writer, encoding=encoding,
-                           address=address, loop=loop)
+
+    conn = cls(reader, writer, encoding=encoding,
+               address=address, parser=parser,
+               loop=loop)
 
     try:
         if password is not None:
-            yield from conn.auth(password)
+            await conn.auth(password)
         if db is not None:
-            yield from conn.select(db)
+            await conn.select(db)
     except Exception:
         conn.close()
-        yield from conn.wait_closed()
+        await conn.wait_closed()
         raise
     return conn
 
 
-class RedisConnection:
+class RedisConnection(AbcConnection):
     """Redis connection."""
 
-    def __init__(self, reader, writer, *, address, encoding=None, loop=None):
+    def __init__(self, reader, writer, *, address, encoding=None,
+                 parser=None, loop=None):
         if loop is None:
             loop = asyncio.get_event_loop()
+        if parser is None:
+            parser = Reader
+        assert callable(parser), (
+            "Parser argument is not callable", parser)
         self._reader = reader
         self._writer = writer
         self._address = address
         self._loop = loop
         self._waiters = deque()
-        self._parser = hiredis.Reader(protocolError=ProtocolError,
-                                      replyError=ReplyError)
-        self._reader_task = async_task(self._read_data(), loop=self._loop)
+        self._reader.set_parser(
+            parser(protocolError=ProtocolError, replyError=ReplyError)
+        )
+        self._reader_task = asyncio.ensure_future(self._read_data(),
+                                                  loop=self._loop)
+        self._close_msg = None
         self._db = 0
         self._closing = False
         self._closed = False
-        self._close_waiter = create_future(loop=self._loop)
+        self._close_waiter = loop.create_future()
         self._reader_task.add_done_callback(self._close_waiter.set_result)
         self._in_transaction = None
         self._transaction_error = None  # XXX: never used?
@@ -133,49 +172,54 @@ class RedisConnection:
     def __repr__(self):
         return '<RedisConnection [db:{}]>'.format(self._db)
 
-    @asyncio.coroutine
-    def _read_data(self):
+    async def _read_data(self):
         """Response reader task."""
         last_error = ConnectionClosedError(
             "Connection has been closed by server")
         while not self._reader.at_eof():
             try:
-                data = yield from self._reader.read(MAX_CHUNK_SIZE)
+                obj = await self._reader.readobj()
             except asyncio.CancelledError:
                 # NOTE: reader can get cancelled from `close()` method only.
                 last_error = RuntimeError('this is unexpected')
                 break
+            except ProtocolError as exc:
+                # ProtocolError is fatal
+                # so connection must be closed
+                if self._in_transaction is not None:
+                    self._transaction_error = exc
+                last_error = exc
+                break
             except Exception as exc:
                 # NOTE: for QUIT command connection error can be received
                 #       before response
                 last_error = exc
                 break
-            self._parser.feed(data)
-            while True:
-                try:
-                    obj = self._parser.gets()
-                except ProtocolError as exc:
-                    # ProtocolError is fatal
-                    # so connection must be closed
-                    self._closing = True
-                    self._loop.call_soon(self._do_close, exc)
-                    if self._in_transaction is not None:
-                        self._transaction_error = exc
-                    return
+            else:
+                if (obj == b'' or obj is None) and self._reader.at_eof():
+                    logger.debug("Connection has been closed by server,"
+                                 " response: %r", obj)
+                    last_error = ConnectionClosedError("Reader at end of file")
+                    break
+
+                if isinstance(obj, MaxClientsError):
+                    last_error = obj
+                    break
+                if self._in_pubsub:
+                    self._process_pubsub(obj)
                 else:
-                    if obj is False:
-                        break
-                    if self._in_pubsub:
-                        self._process_pubsub(obj)
-                    else:
-                        self._process_data(obj)
+                    self._process_data(obj)
         self._closing = True
         self._loop.call_soon(self._do_close, last_error)
 
     def _process_data(self, obj):
         """Processes command results."""
+        assert len(self._waiters) > 0, (type(obj), obj)
         waiter, encoding, cb = self._waiters.popleft()
         if isinstance(obj, RedisError):
+            if isinstance(obj, ReplyError):
+                if obj.args[0].startswith('READONLY'):
+                    obj = ReadOnlyError(obj.args[0])
             _set_exception(waiter, obj)
             if self._in_transaction is not None:
                 self._transaction_error = obj
@@ -198,8 +242,9 @@ class RedisConnection:
 
     def _process_pubsub(self, obj, *, process_waiters=True):
         """Processes pubsub messages."""
-        kind, *pattern, chan, data = obj
+        kind, *args, data = obj
         if kind in (b'subscribe', b'unsubscribe'):
+            chan, = args
             if process_waiters and self._in_pubsub and self._waiters:
                 self._process_data(obj)
             if kind == b'unsubscribe':
@@ -208,6 +253,7 @@ class RedisConnection:
                     ch.close()
             self._in_pubsub = data
         elif kind in (b'psubscribe', b'punsubscribe'):
+            chan, = args
             if process_waiters and self._in_pubsub and self._waiters:
                 self._process_data(obj)
             if kind == b'punsubscribe':
@@ -216,10 +262,14 @@ class RedisConnection:
                     ch.close()
             self._in_pubsub = data
         elif kind == b'message':
+            chan, = args
             self._pubsub_channels[chan].put_nowait(data)
         elif kind == b'pmessage':
-            pattern = pattern[0]
+            pattern, chan = args
             self._pubsub_patterns[pattern].put_nowait((chan, data))
+        elif kind == b'pong':
+            if process_waiters and self._in_pubsub and self._waiters:
+                self._process_data(data or b'PONG')
         else:
             logger.warning("Unknown pubsub message received %r", obj)
 
@@ -233,14 +283,16 @@ class RedisConnection:
           is broken.
         """
         if self._reader is None or self._reader.at_eof():
-            raise ConnectionClosedError("Connection closed or corrupted")
+            msg = self._close_msg or "Connection closed or corrupted"
+            raise ConnectionClosedError(msg)
         if command is None:
             raise TypeError("command must not be None")
-        if None in set(args):
+        if None in args:
             raise TypeError("args must not contain None")
         command = command.upper().strip()
         is_pubsub = command in _PUBSUB_COMMANDS
-        if self._in_pubsub and not is_pubsub:
+        is_ping = command in ('PING', b'PING')
+        if self._in_pubsub and not (is_pubsub or is_ping):
             raise RedisError("Connection in SUBSCRIBE mode")
         elif is_pubsub:
             logger.warning("Deprecated. Use `execute_pubsub` method directly")
@@ -258,7 +310,7 @@ class RedisConnection:
             cb = None
         if encoding is _NOTSET:
             encoding = self._encoding
-        fut = create_future(loop=self._loop)
+        fut = self._loop.create_future()
         self._writer.write(encode_command(command, *args))
         self._waiters.append((fut, encoding, cb))
         return fut
@@ -288,7 +340,7 @@ class RedisConnection:
         cmd = encode_command(command, *(ch.name for ch in channels))
         res = []
         for ch in channels:
-            fut = create_future(loop=self._loop)
+            fut = self._loop.create_future()
             res.append(fut)
             cb = partial(self._update_pubsub, ch=ch)
             self._waiters.append((fut, None, cb))
@@ -309,6 +361,10 @@ class RedisConnection:
         self._reader_task = None
         self._writer = None
         self._reader = None
+
+        if exc is not None:
+            self._close_msg = str(exc)
+
         while self._waiters:
             waiter, *spam = self._waiters.popleft()
             logger.debug("Cancelling waiter %r", (waiter, spam))
@@ -334,9 +390,9 @@ class RedisConnection:
             self._loop.call_soon(self._do_close, None)
         return closed
 
-    @asyncio.coroutine
-    def wait_closed(self):
-        yield from asyncio.shield(self._close_waiter, loop=self._loop)
+    async def wait_closed(self):
+        """Coroutine waiting until connection is closed."""
+        await asyncio.shield(self._close_waiter, loop=self._loop)
 
     @property
     def db(self):
@@ -443,7 +499,3 @@ class RedisConnection:
         """Authenticate to server."""
         fut = self.execute('AUTH', password)
         return wait_ok(fut)
-
-    @asyncio.coroutine
-    def get_atomic_connection(self):
-        return self
