@@ -5,10 +5,10 @@ import aioredis
 async def main():
 
     pool = await aioredis.create_pool(
-        ('localhost', 6379))
+        'redis://localhost')
 
-    async with pool.get() as conn:
-        await conn.set('my-key', 'value')
+    # async with pool.get() as conn:
+    await pool.execute('set', 'my-key', 'value')
 
     await async_with(pool)
     await with_await(pool)
@@ -18,15 +18,15 @@ async def main():
 
 async def async_with(pool):
     async with pool.get() as conn:
-        value = await conn.get('my-key')
+        value = await conn.execute('get', 'my-key')
         print('raw value:', value)
 
 
 async def with_await(pool):
     # This is exactly the same as:
     #   with (yield from pool) as conn:
-    with await pool as conn:
-        value = await conn.get('my-key')
+    with (await pool) as conn:
+        value = await conn.execute('get', 'my-key')
         print('raw value:', value)
 
 
