@@ -4,7 +4,7 @@ import aioredis
 
 async def main():
     redis = await aioredis.create_redis(
-        ('localhost', 6379))
+        'redis://localhost')
 
     async def transaction():
         tr = redis.multi_exec()
@@ -14,6 +14,7 @@ async def main():
         assert result == await asyncio.gather(future1, future2)
         return result
 
+    await transaction()
     redis.close()
     await redis.wait_closed()
 
