@@ -6,65 +6,65 @@ from aioredis import ConnectionClosedError
 
 
 @pytest.mark.run_loop
-def test_multi_exec(redis, loop):
-    yield from redis.delete('foo', 'bar')
+async def test_multi_exec(redis, loop):
+    await redis.delete('foo', 'bar')
 
     tr = redis.multi_exec()
     f1 = tr.incr('foo')
     f2 = tr.incr('bar')
-    res = yield from tr.execute()
+    res = await tr.execute()
     assert res == [1, 1]
-    res2 = yield from asyncio.gather(f1, f2, loop=loop)
+    res2 = await asyncio.gather(f1, f2, loop=loop)
     assert res == res2
 
     tr = redis.multi_exec()
     f1 = tr.incr('foo')
     f2 = tr.incr('bar')
-    yield from tr.execute()
-    assert (yield from f1) == 2
-    assert (yield from f2) == 2
+    await tr.execute()
+    assert (await f1) == 2
+    assert (await f2) == 2
 
     tr = redis.multi_exec()
     f1 = tr.set('foo', 1.0)
     f2 = tr.incrbyfloat('foo', 1.2)
-    res = yield from tr.execute()
+    res = await tr.execute()
     assert res == [True, 2.2]
-    res2 = yield from asyncio.gather(f1, f2, loop=loop)
+    res2 = await asyncio.gather(f1, f2, loop=loop)
     assert res == res2
 
     tr = redis.multi_exec()
     f1 = tr.incrby('foo', 1.0)
-    with pytest.raises_regex(MultiExecError, "increment must be .* int"):
-        yield from tr.execute()
+    with pytest.raises(MultiExecError, match="increment must be .* int"):
+        await tr.execute()
     with pytest.raises(TypeError):
-        yield from f1
+        await f1
 
 
 @pytest.mark.run_loop
-def test_empty(redis):
+async def test_empty(redis):
     tr = redis.multi_exec()
-    res = yield from tr.execute()
+    res = await tr.execute()
     assert res == []
 
 
 @pytest.mark.run_loop
-def test_double_execute(redis):
+async def test_double_execute(redis):
     tr = redis.multi_exec()
-    yield from tr.execute()
+    await tr.execute()
     with pytest.raises(AssertionError):
-        yield from tr.execute()
+        await tr.execute()
     with pytest.raises(AssertionError):
-        yield from tr.incr('foo')
+        await tr.incr('foo')
 
 
 @pytest.mark.run_loop
-def test_connection_closed(redis):
+async def test_connection_closed(redis):
     tr = redis.multi_exec()
     fut1 = tr.quit()
     fut2 = tr.incrby('foo', 1.0)
-    fut3 = tr.connection.execute('INCRBY', 'foo', '1.0')
+    fut3 = tr.incrby('foo', 1)
     with pytest.raises(MultiExecError):
-        yield from tr.execute()
+        await tr.execute()
 
     assert fut1.done() is True
     assert fut2.done() is True
@@ -77,7 +77,7 @@ def test_connection_closed(redis):
     assert not fut3.cancelled()
 
     try:
-        assert (yield from fut1) == b'OK'
+        assert (await fut1) == b'OK'
     except Exception as err:
         assert isinstance(err, (ConnectionClosedError, ConnectionError))
     assert fut2.cancelled() is False
@@ -90,101 +90,101 @@ def test_connection_closed(redis):
 
 
 @pytest.mark.run_loop
-def test_discard(redis):
-    yield from redis.delete('foo')
+async def test_discard(redis):
+    await redis.delete('foo')
     tr = redis.multi_exec()
     fut1 = tr.incrby('foo', 1.0)
     fut2 = tr.connection.execute('MULTI')
     fut3 = tr.connection.execute('incr', 'foo')
 
-    with pytest.raises(ReplyError):
-        yield from tr.execute()
+    with pytest.raises(MultiExecError):
+        await tr.execute()
     with pytest.raises(TypeError):
-        yield from fut1
+        await fut1
     with pytest.raises(ReplyError):
-        yield from fut2
+        await fut2
     # with pytest.raises(ReplyError):
-    res = yield from fut3
+    res = await fut3
     assert res == 1
 
 
 @pytest.mark.run_loop
-def test_exec_error(redis):
+async def test_exec_error(redis):
     tr = redis.multi_exec()
     fut = tr.connection.execute('INCRBY', 'key', '1.0')
     with pytest.raises(MultiExecError):
-        yield from tr.execute()
+        await tr.execute()
     with pytest.raises(ReplyError):
-        yield from fut
+        await fut
 
-    yield from redis.set('foo', 'bar')
+    await redis.set('foo', 'bar')
     tr = redis.multi_exec()
     fut = tr.incrbyfloat('foo', 1.1)
-    res = yield from tr.execute(return_exceptions=True)
+    res = await tr.execute(return_exceptions=True)
     assert isinstance(res[0], ReplyError)
     with pytest.raises(ReplyError):
-        yield from fut
+        await fut
 
 
 @pytest.mark.run_loop
-def test_command_errors(redis):
+async def test_command_errors(redis):
     tr = redis.multi_exec()
     fut = tr.incrby('key', 1.0)
     with pytest.raises(MultiExecError):
-        yield from tr.execute()
+        await tr.execute()
     with pytest.raises(TypeError):
-        yield from fut
+        await fut
 
 
 @pytest.mark.run_loop
-def test_several_command_errors(redis):
+async def test_several_command_errors(redis):
     tr = redis.multi_exec()
     fut1 = tr.incrby('key', 1.0)
     fut2 = tr.rename('bar', 'bar')
     with pytest.raises(MultiExecError):
-        yield from tr.execute()
+        await tr.execute()
     with pytest.raises(TypeError):
-        yield from fut1
+        await fut1
     with pytest.raises(ValueError):
-        yield from fut2
+        await fut2
 
 
 @pytest.mark.run_loop
-def test_error_in_connection(redis):
-    yield from redis.set('foo', 1)
+async def test_error_in_connection(redis):
+    await redis.set('foo', 1)
     tr = redis.multi_exec()
     fut1 = tr.mget('foo', None)
     fut2 = tr.incr('foo')
     with pytest.raises(MultiExecError):
-        yield from tr.execute()
+        await tr.execute()
     with pytest.raises(TypeError):
-        yield from fut1
-    yield from fut2
+        await fut1
+    await fut2
 
 
 @pytest.mark.run_loop
-def test_watch_unwatch(redis):
-    res = yield from redis.watch('key')
+async def test_watch_unwatch(redis):
+    res = await redis.watch('key')
     assert res is True
-    res = yield from redis.watch('key', 'key')
+    res = await redis.watch('key', 'key')
     assert res is True
 
     with pytest.raises(TypeError):
-        yield from redis.watch(None)
+        await redis.watch(None)
     with pytest.raises(TypeError):
-        yield from redis.watch('key', None)
+        await redis.watch('key', None)
     with pytest.raises(TypeError):
-        yield from redis.watch('key', 'key', None)
+        await redis.watch('key', 'key', None)
 
-    res = yield from redis.unwatch()
+    res = await redis.unwatch()
     assert res is True
 
 
 @pytest.mark.run_loop
-def test_encoding(redis):
-    res = yield from redis.set('key', 'value')
+async def test_encoding(redis):
+    res = await redis.set('key', 'value')
     assert res is True
-    res = yield from redis.hmset(
+    res = await redis.hmset(
         'hash-key', 'foo', 'val1', 'bar', 'val2')
     assert res is True
 
@@ -192,23 +192,23 @@ def test_encoding(redis):
     fut1 = tr.get('key')
     fut2 = tr.get('key', encoding='utf-8')
     fut3 = tr.hgetall('hash-key', encoding='utf-8')
-    yield from tr.execute()
-    res = yield from fut1
+    await tr.execute()
+    res = await fut1
     assert res == b'value'
-    res = yield from fut2
+    res = await fut2
     assert res == 'value'
-    res = yield from fut3
+    res = await fut3
     assert res == {'foo': 'val1', 'bar': 'val2'}
 
 
 @pytest.mark.run_loop
-def test_global_encoding(redis, create_redis, server, loop):
-    redis = yield from create_redis(
+async def test_global_encoding(redis, create_redis, server, loop):
+    redis = await create_redis(
         server.tcp_address,
         loop=loop, encoding='utf-8')
-    res = yield from redis.set('key', 'value')
+    res = await redis.set('key', 'value')
     assert res is True
-    res = yield from redis.hmset(
+    res = await redis.hmset(
         'hash-key', 'foo', 'val1', 'bar', 'val2')
     assert res is True
 
@@ -216,35 +216,56 @@ def test_global_encoding(redis, create_redis, server, loop):
     fut1 = tr.get('key')
     fut2 = tr.get('key', encoding='utf-8')
     fut3 = tr.hgetall('hash-key', encoding='utf-8')
-    yield from tr.execute()
-    res = yield from fut1
+    await tr.execute()
+    res = await fut1
     assert res == 'value'
-    res = yield from fut2
+    res = await fut2
     assert res == 'value'
-    res = yield from fut3
+    res = await fut3
     assert res == {'foo': 'val1', 'bar': 'val2'}
 
 
 @pytest.mark.run_loop
-def test_transaction__watch_error(redis, create_redis, server, loop):
-    other = yield from create_redis(
+async def test_transaction__watch_error(redis, create_redis, server, loop):
+    other = await create_redis(
         server.tcp_address, loop=loop)
 
-    ok = yield from redis.set('foo', 'bar')
+    ok = await redis.set('foo', 'bar')
     assert ok is True
 
-    ok = yield from redis.watch('foo')
+    ok = await redis.watch('foo')
     assert ok is True
 
-    ok = yield from other.set('foo', 'baz')
+    ok = await other.set('foo', 'baz')
     assert ok is True
 
     tr = redis.multi_exec()
     fut1 = tr.set('foo', 'foo')
     fut2 = tr.get('bar')
     with pytest.raises(MultiExecError):
-        yield from tr.execute()
+        await tr.execute()
     with pytest.raises(WatchVariableError):
-        yield from fut1
+        await fut1
     with pytest.raises(WatchVariableError):
-        yield from fut2
+        await fut2
+
+
+@pytest.mark.run_loop
+async def test_multi_exec_and_pool_release(redis):
+    # Test the case when pool connection is released before
+    # `exec` result is received.
+
+    slow_script = """
+    local a = tonumber(redis.call('time')[1])
+    local b = a + 1
+    while (a < b)
+    do
+        a = tonumber(redis.call('time')[1])
+    end
+    """
+
+    tr = redis.multi_exec()
+    fut1 = tr.eval(slow_script)
+    ret, = await tr.execute()
+    assert ret is None
+    assert (await fut1) is None
